@@ -146,7 +146,12 @@ func TestGeneratePackage(t *testing.T) {
 				})
 			assert.NoError(t, err)
 
-			expectedFiles, err := test.LoadFiles(filepath.Join(testDir, tt.schemaDir), "go", tt.expectedFiles)
+			dir := filepath.Join(testDir, tt.schemaDir)
+			lang := "go"
+
+			test.RewriteFilesWhenPulumiAccept(t, dir, lang, files)
+
+			expectedFiles, err := test.LoadFiles(filepath.Join(testDir, tt.schemaDir), lang, tt.expectedFiles)
 			assert.NoError(t, err)
 			test.ValidateFileEquality(t, files, expectedFiles)
 			test.CheckAllFilesGenerated(t, files, expectedFiles)
