@@ -321,6 +321,15 @@ type Resource struct {
 	Language map[string]interface{}
 	// IsComponent indicates whether the resource is a ComponentResource.
 	IsComponent bool
+	// Methods is the list of methods for the resource.
+	Methods []*Method
+}
+
+type Method struct {
+	// Name is the name of the method.
+	Name string
+	// Function is the function definition for the method.
+	Function *Function
 }
 
 // Function describes a Pulumi function.
@@ -339,6 +348,8 @@ type Function struct {
 	DeprecationMessage string
 	// Language specifies additional language-specific data about the function.
 	Language map[string]interface{}
+	// IsMethod indicates whether the function is a method of a resource.
+	IsMethod bool
 }
 
 // Package describes a Pulumi package.
@@ -767,6 +778,8 @@ type ResourceSpec struct {
 	Language map[string]json.RawMessage `json:"language,omitempty"`
 	// IsComponent indicates whether the resource is a ComponentResource.
 	IsComponent bool `json:"isComponent,omitempty"`
+	// Methods maps method names to functions in this schema.
+	Methods map[string]string `json:"methods,omitempty"`
 }
 
 // FunctionSpec is the serializable form of a function description.
@@ -893,21 +906,21 @@ func importSpec(spec PackageSpec, languages map[string]Language, loader Loader)
 		return nil, errors.Wrap(err, "binding config")
 	}
 
-	provider, err := bindProvider(spec.Name, spec.Provider, types)
+	functions, functionTable, err := bindFunctions(spec.Functions, types)
+	if err != nil {
+		return nil, errors.Wrap(err, "binding functions")
+	}
+
+	provider, err := bindProvider(spec.Name, spec.Provider, types, functionTable)
 	if err != nil {
 		return nil, errors.Wrap(err, "binding provider")
 	}
 
-	resources, resourceTable, err := bindResources(spec.Resources, types)
+	resources, resourceTable, err := bindResources(spec.Resources, types, functionTable)
 	if err != nil {
 		return nil, errors.Wrap(err, "binding resources")
 	}
 
-	functions, functionTable, err := bindFunctions(spec.Functions, types)
-	if err != nil {
-		return nil, errors.Wrap(err, "binding functions")
-	}
-
 	// Build the type list.
 	var typeList []Type
 	for _, t := range types.resources {
@@ -1605,12 +1618,49 @@ func bindTypes(pkg *Package, complexTypes map[string]ComplexTypeSpec, loader Loa
 	return typs, nil
 }
 
+func bindMethods(resourceToken string, methods map[string]string,
+	functionTable map[string]*Function) ([]*Method, error) {
+
+	names := make([]string, 0, len(methods))
+	for name := range methods {
+		names = append(names, name)
+	}
+	sort.Strings(names)
+
+	result := make([]*Method, 0, len(methods))
+	for _, name := range names {
+		token := methods[name]
+		function, ok := functionTable[token]
+		if !ok {
+			return nil, errors.Errorf("unknown function %s for method %s", token, name)
+		}
+		if function.IsMethod {
+			return nil, errors.Errorf("function %s for method %s is already a method", token, name)
+		}
+		idx := strings.LastIndex(function.Token, "/")
+		if idx == -1 || function.Token[:idx] != resourceToken {
+			return nil, errors.Errorf("invalid function token format %s for method %s", token, name)
+		}
+		if function.Inputs == nil || function.Inputs.Properties == nil || len(function.Inputs.Properties) == 0 ||
+			function.Inputs.Properties[0].Name != "__self__" {
+			return nil, errors.Errorf("function %s for method %s is missing __self__ parameter", token, name)
+		}
+		function.IsMethod = true
+		result = append(result, &Method{
+			Name:     name,
+			Function: function,
+		})
+	}
+	return result, nil
+}
+
 func bindConfig(spec ConfigSpec, types *types) ([]*Property, error) {
 	properties, _, err := types.bindProperties(spec.Variables, spec.Required, nil)
 	return properties, err
 }
 
-func bindResource(token string, spec ResourceSpec, types *types) (*Resource, error) {
+func bindResource(token string, spec ResourceSpec, types *types,
+	functionTable map[string]*Function) (*Resource, error) {
 	if len(spec.Plain) > 0 {
 		return nil, errors.New("plain cannot be specified on resources")
 	}
@@ -1628,6 +1678,17 @@ func bindResource(token string, spec ResourceSpec, types *types) (*Resource, err
 		return nil, errors.Wrap(err, "failed to bind properties")
 	}
 
+	methods, err := bindMethods(token, spec.Methods, functionTable)
+	if err != nil {
+		return nil, errors.Wrap(err, "failed to bind methods")
+	}
+
+	for _, method := range methods {
+		if _, ok := spec.Properties[method.Name]; ok {
+			return nil, errors.Errorf("property and method have the same name %s", method.Name)
+		}
+	}
+
 	var stateInputs *ObjectType
 	if spec.StateInputs != nil {
 		si, err := types.bindObjectType(token+"Args", *spec.StateInputs, false /*supportsPlainProperties*/)
@@ -1658,11 +1719,13 @@ func bindResource(token string, spec ResourceSpec, types *types) (*Resource, err
 		DeprecationMessage: spec.DeprecationMessage,
 		Language:           language,
 		IsComponent:        spec.IsComponent,
+		Methods:            methods,
 	}, nil
 }
 
-func bindProvider(pkgName string, spec ResourceSpec, types *types) (*Resource, error) {
-	res, err := bindResource("pulumi:providers:"+pkgName, spec, types)
+func bindProvider(pkgName string, spec ResourceSpec, types *types,
+	functionTable map[string]*Function) (*Resource, error) {
+	res, err := bindResource("pulumi:providers:"+pkgName, spec, types, functionTable)
 	if err != nil {
 		return nil, errors.Wrap(err, "error binding provider")
 	}
@@ -1671,15 +1734,22 @@ func bindProvider(pkgName string, spec ResourceSpec, types *types) (*Resource, e
 	// Since non-primitive provider configuration is currently JSON serialized, we can't handle it without
 	// modifying the path by which it's looked up. As a temporary workaround to enable access to config which
 	// values which are primitives, we'll simply remove any properties for the provider resource which are not
-	// here, before we generate the provider code.
-	var primitiveProperties []*Property
+	// strings, or types with an underlying type of string, before we generate the provider code.
+	var stringProperties []*Property
 	for _, prop := range res.Properties {
-		if prop.Type != stringType {
-			continue
+		if tokenType, isTokenType := prop.Type.(*TokenType); isTokenType {
+			if tokenType.UnderlyingType != stringType {
+				continue
+			}
+		} else {
+			if prop.Type != stringType {
+				continue
+			}
 		}
-		primitiveProperties = append(primitiveProperties, prop)
+
+		stringProperties = append(stringProperties, prop)
 	}
-	res.Properties = primitiveProperties
+	res.Properties = stringProperties
 
 	types.resources[res.Token] = &ResourceType{
 		Token:    res.Token,
@@ -1689,11 +1759,12 @@ func bindProvider(pkgName string, spec ResourceSpec, types *types) (*Resource, e
 	return res, nil
 }
 
-func bindResources(specs map[string]ResourceSpec, types *types) ([]*Resource, map[string]*Resource, error) {
+func bindResources(specs map[string]ResourceSpec, types *types,
+	functionTable map[string]*Function) ([]*Resource, map[string]*Resource, error) {
 	resourceTable := map[string]*Resource{}
 	var resources []*Resource
 	for token, spec := range specs {
-		res, err := bindResource(token, spec, types)
+		res, err := bindResource(token, spec, types, functionTable)
 		if err != nil {
 			return nil, nil, errors.Wrapf(err, "error binding resource %v", token)
 		}
